@@ -1,5 +1,5 @@
 #
-# $Id: Storable.pm,v 14.2 2000/08/15 20:51:51 wpm Exp $
+# $Id: Storable.pm,v 15.1 2000/08/16 00:58:41 wpm Exp $
 #
 # (c) 1999, 2000 Morgan Stanley Dean Witter and Co.
 # See ..../src/LICENSE for terms of distribution.
@@ -17,8 +17,9 @@ use Storable qw(nfreeze thaw);
 
 use MQSeries::Message;
 
-use vars qw(@ISA);
+use vars qw(@ISA $VERSION);
 
+$VERSION = '1.12';
 @ISA = qw(MQSeries::Message);
 
 #
